/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2009 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 *
 *******************************************************************************
 *
 * $Id: ShutterFormat.java,v d60681e7fc1d 2009/09/17 23:59:23 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.metadata.text;

import java.text.DecimalFormat;
import java.text.FieldPosition;
import java.text.ParseException;
import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ShutterFormat.java,v d60681e7fc1d 2009/09/17 23:59:23 fabrizio $
 *
 ******************************************************************************/
public class ShutterFormat extends DecimalFormat
  {
    private static final long serialVersionUID = 674556562345876458L;

    @Override
    @Nonnull
    public StringBuffer format (@Nonnegative double d,
                                @Nonnull final StringBuffer toAppendTo,
                                @Nonnull final FieldPosition pos)
      {
        String s;

        if (d < 0)
          { 
            s = "";
          }

        else if (d >= 1)
          {
            s = Double.toString(d).replaceAll("\\.0$", "");
          }
        
//        else if (d > 0.5)
//          {
//            int numerator = 0;
//            
//            for (int denominator = 10; ; denominator *= 10)
//              {
//                double n = d * denominator;
//                numerator = (int)n;
//                
//                if (numerator == n)
//                  {
//                    s = Long.toString(numerator) + "/" + Long.toString(denominator);
//                    break;  
//                  }
//              }  
//          }

        else
          {
            final double x = 100;
            final double d2 = Math.round(x / d) / x;
            s = "1/" + Double.toString(d2).replaceAll("\\.0$", "");
          }

        toAppendTo.append(s + " sec");

        return toAppendTo;
      }

    @Override
    @Nonnull
    public Number parse (@CheckForNull String s)
      throws ParseException
      {
        if (s != null)
          {
            s = s.replaceAll(" sec$", "").trim();
          }
        
        if ((s == null) || s.equals(""))
          {
            return Double.valueOf(-1); // FIXME
          }

        if (s.startsWith("1/"))
          {
            return Double.valueOf(1 / Double.parseDouble(s.substring(2)));
          }

        else
          {
            return Double.valueOf(s);
          }
      }
  }
